import datetime
import xbmc
from xbmcgui import Dialog, DialogProgress, ListItem

from resources.lib.const import STRINGS, SETTINGS, LANG, quality_map, lang_code_gui, audio_channels, ROUTE
from resources.lib.gui import MediaItem
from resources.lib.storage.settings import settings
from resources.lib.utils.streams import resolution_to_quality
from resources.lib.utils.kodiutils import show_input, get_string, convert_size, make_table, \
    append_list_items_to_nested_list_items, convert_bitrate


class DialogRenderer:

    @staticmethod
    def search():
        search_value = show_input(get_string(30207))
        if search_value:
            return search_value

    @staticmethod
    def choose_video_stream(streams):
        stream_labels = []
        streams.sort(key=lambda d: d.get('size', 0), reverse=settings.as_int(SETTINGS.FILE_SIZE_SORT))
        audio_info_list = []
        for stream in streams:
            # Fix audio string that begins with the comma.
            audio_info = []
            for audio in stream.get('audio'):
                lang_code = lang_code_gui.get(audio.get('language', '').lower()) or ''
                channels = audio.get('channels')
                channels = audio_channels.get(channels, channels)
                audio_info.append(STRINGS.AUDIO_INFO.format(audio.get('codec'),
                                                            format(channels, '.1f'),
                                                            lang_code))
            audio_info_list.append(' '.join(audio_info))
            video = stream.get('video')
            bit_rate = ''
            duration = ''
            size = ''
            quality = ''
            if len(video) > 0:
                video = video[0]
                quality = STRINGS.STREAM_TITLE_BRACKETS.format(
                    resolution_to_quality(video))
            else:
                video = {}

            if stream.get('size'):
                size = STRINGS.BOLD.format(convert_size(stream.get('size')))

            if video.get('duration') and settings.as_bool(SETTINGS.SHOW_DURATION):
                duration = STRINGS.BOLD.format(str(datetime.timedelta(seconds=int(video.get('duration')))))

            if video.get('duration') and stream.get('size') and settings.as_bool(SETTINGS.SHOW_BITRATE):
                bit_rate = stream.get('size') / video.get('duration') * 8
                bit_rate = STRINGS.STREAM_BITRATE_BRACKETS.format(convert_bitrate(bit_rate))

            codec = STRINGS.STREAM_TITLE_BRACKETS.format(video.get('codec')) if settings.as_bool(
                SETTINGS.SHOW_CODEC) else ''

            stream_labels.append([quality, codec, size, duration, bit_rate])

        table = make_table(stream_labels)
        table = append_list_items_to_nested_list_items(table, audio_info_list)

        ret = Dialog().select(get_string(LANG.CHOOSE_STREAM), [STRINGS.TABLE_SPACES.join(item) for item in table])
        if ret < 0:
            return None
        return streams[ret]

    @staticmethod
    def keyboard(title, hidden=False):
        keyboard = xbmc.Keyboard('', title, hidden)
        keyboard.doModal()
        if keyboard.isConfirmed():
            return keyboard.getText().strip()
        else:
            return None

    @staticmethod
    def ok(heading, *args, **kwargs):
        return Dialog().ok(heading, *args, **kwargs)

    @staticmethod
    def ok_multi_line(heading, lines):
        return DialogRenderer.ok(heading, STRINGS.NEW_LINE.join(lines))

    @staticmethod
    def progress(heading, *args, **kwargs):
        d = DialogProgress()
        d.create(heading, *args, **kwargs)
        return d

    @staticmethod
    def select(heading, choices):
        return Dialog().select(heading, choices)

    @staticmethod
    def yesno(heading, *args, **kwargs):
        return Dialog().yesno(heading, *args, **kwargs)
